import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "components"
import "./view"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Форма авторизации
 */
BasicView {
    id: root
    color: Style.parameter("color.lightBackground")
    implicitWidth: primaryScreen.geometry.width / scale
    withBottomPanel: false

    property bool inputPassMode: false
    property BasicTextField eventReceiver: inputField
    property DialogEventFilter eventFilter: DialogEventFilter {
        id: eventFilter
        onKeyboardData: (data, rawData, source) => {
            inputField.text = ""
            handler.input(data, rawData, source)
        }
        onOk: {
            if (!inputField.text && !inputPassMode)
                return
            inputPassMode = false
            var password = inputField.text
            inputField.text = ""
            handler.registerByPassword(password)
        }
        onCancel: {
            if (inputPassMode) {
                inputPassMode = false
                return
            }
            inputField.text = ""
        }
    }
    contentItem: Item {
        BasicButton {
            id: usersButton
            Style.class: "authUsers"
            anchors.right: parent.right
            anchors.top: parent.top
            anchors.margins: 24
            height: 68
            width: 68
            radius: width / 2
            icon: Style.parameter("icon.authUsers")
            onClicked: usersList.open()
        }
        FullLogo {
            id: logo
            anchors.horizontalCenter: parent.horizontalCenter
        }
        BasicTextField {
            id: inputField
            anchors.horizontalCenter: parent.horizontalCenter
            Style.class: "authenticationPassword"
            height: 32
            echoMode: TextField.Password
            horizontalAlignment: TextField.AlignHCenter
            hideCursor: true
            fixedCursorPosition: true
            onActiveFocusChanged: focus = false
            Tr on placeholderText {
                code: "authorizationHint"
                defaultMessage: "Воспользуйтесь картой или введите пароль"
            }
            placeholder { verticalAlignment: Text.AlignVCenter; horizontalAlignment: Text.AlignHCenter }
        }
        InlineKeyboard {
            id: keyboard
            anchors.horizontalCenter: parent.horizontalCenter
            y: 368
            visible: inputPassMode
            width: layoutType !== "numbers" ? 960 : 256
            height: 296
            layoutType: "numbers"
            defaultLocale: "ru_RU"
            onEnter: eventFilter.ok()
        }
        BasicButton {
            anchors.bottom: keyboard.bottom
            anchors.left: keyboard.right
            anchors.leftMargin: 10
            visible: keyboard.visible && keyboard.layoutType === "numbers"
            opacity: keyboard.opacity
            width: 124
            height: 144
            Style.class: "keyboardEnter"
            Tr { code: "keyboardEnterButton"; defaultMessage: "Ввод" }
            onClicked: eventFilter.ok()
        }
        BasicLabel {
            id: cashId
            anchors.horizontalCenter: parent.horizontalCenter
            Style.class: "startupCash"
            Tr { code: "cashCode"; defaultMessage: "Касса %1"; args: [Session.cashCode] }
        }
        MouseArea {
            anchors.fill: parent
            enabled: !inputPassMode
            onClicked: inputPassMode = true
        }
    }
    BasicButton {
        Style.class: "employeeMarkButton"
        anchors {
            bottom: parent.bottom
            right: parent.right
            rightMargin: 24
            bottomMargin: 24
        }
        // в прототипе ширина 124 высота 144, пришлось расширить кнопку, т.к. контейнер с иконкой не дает отобразить текст без переноса
        // после решения проблемы позиционирования иконок на кнопке надо вернуть размер
        width: 154
        height: 104
        icon: Style.parameter("icon.employeeMark.icon")
        iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
        textLayout { fillWidth: true; fillHeight: true }
        verticalAlignment: Text.AlignBottom
        Tr { code: "employeeMarkButton"; defaultMessage: "Отметка сотрудника" }
        onClicked: handler.markEmployee()
        visible: !(keyboard.visible && keyboard.layoutType === "letters") && Config.getLogical("plugins:employeemark", false)
    }
    states: [
        State {
            name: "default"; when: !keyboard.visible
            PropertyChanges { target: logo; y: 208; width: 280; height: 226 }
            PropertyChanges { target: inputField; y: 536 }
            PropertyChanges { target: keyboard; opacity: 0 }
            PropertyChanges { target: cashId; y: 624 }
            PropertyChanges { target: usersButton; opacity: 0 }
        },
        State {
            name: "keyboard"; when: keyboard.visible
            PropertyChanges { target: logo; y: 80; width: 176; height: 147 }
            PropertyChanges { target: inputField; y: 288 }
            PropertyChanges { target: keyboard; opacity: 1 }
            PropertyChanges { target: cashId; y: 704 }
            PropertyChanges { target: usersButton; opacity: 1 }
        }
    ]
    state: "default"
    transitions: [
        Transition {
            SequentialAnimation {
                ParallelAnimation {
                    id: parallel
                    readonly property int baseSpeed: 200
                    SmoothedAnimation {
                        target: logo; property: "y"; velocity: parallel.baseSpeed
                    }
                    SmoothedAnimation {
                        target: logo; property: "width";
                        velocity: parallel.baseSpeed * 0.81
                    }
                    SmoothedAnimation {
                        target: logo; property: "height"; velocity: parallel.baseSpeed * 0.61
                    }
                    SmoothedAnimation {
                        target: inputField; property: "y"; velocity: parallel.baseSpeed * 1.94
                    }
                    SmoothedAnimation {
                        target: cashId; property: "y"; velocity: parallel.baseSpeed * 0.63
                    }
                    NumberAnimation {
                        target: usersButton; property: "opacity"; duration: 500
                    }
                }
                NumberAnimation { target: keyboard; property: "opacity"; duration: 300}
            }
        }
    ]

    Popup {
        id: usersList
        width: root.implicitWidth
        height: root.implicitHeight
        scale: root.scale
        transformOrigin: Item.TopLeft
        padding: 0
        UsersList {
            anchors.fill: parent
            onClose: usersList.close()
        }
    }

    AuthenticationHandler { id: handler }
}
