import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes
import Qt.labs.qmlmodels

import "components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Диалог для многострочного ввода текста
 */
FullscreenDialog {
    id: root

    padding: 0
    leftPadding: 24
    rightPadding: 24
    property var fieldsById: ({})
    property var rowComponents: ({})
    property real delegateHeight: 64
    property real delegateSpacing: 12
    onClosed: {
        if (eventReceiver)
            eventReceiver.focus = false
        eventReceiver = null
    }

    property BasicTextField eventReceiver: null
    property DialogEventFilter eventFilter: DialogEventFilter {
        onKeyboardData: setData(data, rawData, source)
        onOk: accept()
        onCancel: reject()
    }
    Connections {
        target: ActionQueueController
        function onInputData(data, rawData, source) { setData(data, rawData, source) }
    }
    // Получить поле для ввода
    function fieldForInput() {
        for (let i = 0; i < params.fields.length; ++i) {
            let field = fieldsById[params.fields[i].fieldId]
            if (field.fieldType === InputMultiTextModel.Choice
                    || field.fieldType === InputMultiTextModel.Check || !params.fields[i].editable)
                continue
            return field
        }
        return null
    }
    // Обработать ввод данных
    function setData(data, rawData, source) {
        let reciever = eventReceiver ? eventReceiver : fieldForInput()
        if (!reciever)
            return

        if (!handler.testAllowDevices(source)) {
            reciever.text = ""
            return
        }

        // Устанавливам маску до применения текста, если нужно
        if (!reciever.inputMask && reciever.mask)
            reciever.inputMask = reciever.mask

        if (!params.fieldDistributionRegexp
                || !handler.distributeData(data, params.fieldDistributionRegexp, fieldsById)) {
            switch (reciever.fieldType) {
            case InputMultiTextModel.Choice:
                break
            case InputMultiTextModel.DateTime:
                reciever.text = Date.fromLocaleString(Qt.locale(), data, reciever.regexp).toLocaleString(Qt.locale(), reciever.dateFormat)
                break
            case InputMultiTextModel.String:
            case InputMultiTextModel.Date:
            default:
                reciever.text = data
                break;
            }
        }

        if (params.returnOnValidScan)
            accept()
    }

    // Базовый делегат для поля ввода
    component InputDelegate: BasicTextField {
        id: inputDelegate
        objectName: field.fieldId
        readOnly: !field.editable
        Style.class: validValue ? "inputMultiText" : "inputMultiTextInvalid"
        padding: 12

        function addComponentToRow() {
            if (!rowComponents[fieldRow])
                rowComponents[fieldRow] = {}
            if (!rowComponents[fieldRow][fieldColumn])
                rowComponents[fieldRow][fieldColumn] = this
        }

        x: {
            addComponentToRow()
            let res = 0
            for (let i = 0; i < fieldColumn; ++i)
                res += rowComponents[fieldRow][i].width + delegateSpacing
            return res
        }
        y: fieldRow * (delegateHeight + delegateSpacing)

        height: delegateHeight
        width: {
            addComponentToRow()
            if (fixedWidth)
                return maskWidth + leftPadding + rightPadding + imageSize + imageSpacing
            /**
             * Поля без маски занимают все свободное пространство.
             * Если таких полей больше 1, то они делят пространство поровну
             */
            let w = parent.width
            let rowLen = Object.values(rowComponents[fieldRow]).length
            if (rowLen < 2)
                return w
            let fillCount = 0
            for (let i = 0; i < rowLen; ++i) {
                if (rowComponents[fieldRow][i].fixedWidth)
                    w = w - rowComponents[fieldRow][i].width - delegateSpacing
                else
                    fillCount++
            }
            w = w / fillCount - delegateSpacing * (fillCount - 1) / fillCount
            return w
        }

        property real imageSize: 30
        property real imageSpacing: 8
        property string mask: ""
        property string defaultText: field.defaultValue
        property bool noTries: true
        function callValidator() {}
        onTextChanged: {
            noTries = false
            callValidator()
        }
        property int fieldType: type
        property string regexp: field.regexp
        property string dateFormat: field.dateFormat

        // Для полей с маской, ширина поля определяется длиной текста маски
        TextMetrics {
            id: metric
            text: inputDelegate.mask
            font: inputDelegate.font
        }
        property alias maskWidth: metric.width
        property bool fixedWidth: maskWidth

        // Определить позицию символа заполнителя
        function blankCharPos() {
            if (!inputMask)
                return -1
            let blankChar = inputMask[inputMask.length - 1]
            if (blankChar === ';')
                blankChar = ' '
            let cursorPos = -1
            for (let i = 0; i < displayText.length; ++i) {
                if (displayText[i] === blankChar) {
                    cursorPos = i
                    break
                }
            }
            return cursorPos
        }
        onFocusChanged: {
            if (focus) {
                eventReceiver = this
                if (!text)
                    text = defaultText
                if (mask && !inputMask)
                    inputMask = mask
            } else {
                eventReceiver = null
            }
        }

        Image {
            opacity: inputDelegate.text.length > 0 ? 1 : 0
            anchors{ right: parent.right; rightMargin: parent.rightPadding; verticalCenter: parent.verticalCenter }
            width: imageSize; height: imageSize
            source: Style.parameter("icon.textFieldClear")
            Component.onCompleted: mainWindow.scaleImage(this)
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    inputDelegate.focus = false
                    inputDelegate.inputMask = ""
                    inputDelegate.clear()
                }
            }
            Behavior on opacity { NumberAnimation { duration: 250 } }
        }
        Tr on placeholderText { tr: field.description }
        onCursorPositionChanged: {
            let pos = blankCharPos()
            if (pos >= 0 && pos !== cursorPosition)
                cursorPosition = pos
        }
        Component.onCompleted: {
            if (!root.fieldsById)
                root.fieldsById = {}
            root.fieldsById[objectName] = this
        }
    }
    // Делегат для ввода произвольного текста
    component StringDelegate: InputDelegate {
        RegExValidator { id: rev; regularExpression: field.regexp ? Utils.anchoredPattern(field.regexp) : new RegExp() }
        function callValidator() { rev.validate(text) }
        property bool validatorResult: rev.state === RegExValidator.Acceptable
        property bool validValue: validatorResult && text || !field.required || !field.editable || noTries
        mask: field.inputMask
    }
    // Делегат для ввода даты
    component DateDelegate: InputDelegate {
        mask: "99.99.9999;_"
        property string dateFormat: "dd.MM.yyyy"
        property bool validatorResult: dv.state === RegExValidator.Acceptable
        property bool validValue: acceptableInput && validatorResult && text
                    || (!field.required && (text == ".." || !text)) || !field.editable || noTries
        DateTimeValidator {
            id: dv
            minimumDate: field.minDate
            maximumDate: field.maxDate
            dateTimeFormat: dateFormat
        }
        function callValidator() { dv.validate(text) }
    }
    // Делегат для ввода даты и времени
    component DateTimeDelegate: InputDelegate {
        mask: Utils.dateTimeFormatToMask(field.dateFormat, '_')
        defaultText: (new Date()).toLocaleString(Locale.LongFormat, field.dateFormat)
        DateTimeValidator { id: dtv; dateTimeFormat: field.dateFormat }
        function callValidator() { dtv.validate(text) }
        property bool validatorResult: dtv.state === RegExValidator.Acceptable
        property bool validValue: acceptableInput && validatorResult && text || !field.required || !field.editable || noTries
    }
    // Делегат выбора с помощью RadioButton
    component ChoiceDelegate: Item {
        objectName: field.fieldId
        height: delegateHeight
        width: buttons.implicitWidth

        x: {
            if (!rowComponents[fieldRow])
                rowComponents[fieldRow] = {}
            rowComponents[fieldRow][fieldColumn] = this
            let res = 0
            for (let i = 0; i < fieldColumn; ++i)
                res += rowComponents[fieldRow][i].width + delegateSpacing
            return res
        }
        y: fieldRow * (delegateHeight + delegateSpacing)

        property bool noTries: false
        property bool validValue: true
        property int fieldType: type
        property int choiceValue: group.checkedButton ? group.checkedButton.buttonIndex : -1
        property bool fixedWidth: true
        ButtonGroup { id: group; buttons: buttons.children }
        RowLayout {
            id: buttons
            spacing: 0
            anchors.verticalCenter: parent.verticalCenter
            Repeater {
                id: repeater
                model: field.labels.count
                delegate: BasicRadioButton {
                    Style.class: "inputMultiText"
                    checked: field.defaultValue ? index === Number(field.defaultValue) : index === 0
                    property int buttonIndex: index
                    Tr { tr: field.labels.value(index) }
                    Line {
                        visible: index !== 0
                        anchors.right: parent.left
                        height: parent.height
                        strokeStyle: ShapePath.DashLine
                        direction: LineDirection.Vertical
                        dashPattern: [2, 3]
                        strokeWidth: 1
                        color: Style.parameter("color.gray")
                    }
                }
            }
        }
        Component.onCompleted: root.fieldsById[objectName] = this
    }
    // Делегат подтверждения
    component CheckDelegate: BasicCheckBox {
        objectName: field.fieldId
        height: delegateHeight

        x: {
            if (!rowComponents[fieldRow])
                rowComponents[fieldRow] = {}
            rowComponents[fieldRow][fieldColumn] = this
            let res = 0
            for (let i = 0; i < fieldColumn; ++i)
                res += rowComponents[fieldRow][i].width + delegateSpacing
            return res
        }
        y: fieldRow * (delegateHeight + delegateSpacing)

        property bool noTries: false
        property bool validValue: true
        property int fieldType: type
        property bool fixedWidth: true
        Tr { tr: field.description }
        Component.onCompleted: root.fieldsById[objectName] = this
    }

    contentItem: ColumnLayout {
        spacing: 0
        RowLayout {
            BasicLabel {
                Style.class: "inputMultiTextTitle"
                Layout.preferredWidth: contentItem.width / 2
                Layout.topMargin: 16
                Tr { tr: params.title }
            }
            TopPanel {
                Layout.preferredWidth: contentItem.width / 2
                hideUserName: true
                sideMenuButton.anchors.rightMargin: -9
            }
        }
        Item { Layout.preferredHeight: delegateSpacing * 2 }
        BasicLabel {
            id: descripton
            Style.class: "inputMultiTextDescription"
            Layout.fillWidth: true
            visible: !params.hideDescription
            Tr { tr: params.description }
        }
        Item { visible: descripton.visible; Layout.preferredHeight: delegateSpacing }
        Item {
            Layout.fillWidth: true
            Layout.fillHeight: true
            Repeater {
                model: InputMultiTextModel { fields: params.fields }
                delegate: DelegateChooser {
                    role: "type"
                    DelegateChoice {
                        roleValue: InputMultiTextModel.String
                        delegate: StringDelegate {}
                    }
                    DelegateChoice {
                        roleValue: InputMultiTextModel.Date
                        delegate: DateDelegate {}
                    }
                    DelegateChoice {
                        roleValue: InputMultiTextModel.DateTime
                        delegate: DateTimeDelegate {}
                    }
                    DelegateChoice {
                        roleValue: InputMultiTextModel.Choice
                        delegate: ChoiceDelegate {}
                    }
                    DelegateChoice {
                        roleValue: InputMultiTextModel.Check
                        delegate: CheckDelegate {}
                    }
                }
            }
        }
    }

    footer: Rectangle {
        height: 116
        color: Style.parameter("color.modalBackground")
        RowLayout {
            anchors {
                right: parent.right
                verticalCenter: parent.verticalCenter
                rightMargin: 24
            }
            spacing: 8
            BasicButton {
                Layout.preferredHeight: 68
                Layout.preferredWidth: 120
                textLayout {
                    fillWidth: true
                    fillHeight: true
                }
                horizontalAlignment: Text.AlignLeft
                verticalAlignment: Text.AlignBottom
                visible: !params.hideCancelButton
                Style.class: "inputMultiTextCancel"
                Tr { code: "cancelButton"; defaultMessage: "Отмена" }
                onClicked: reject()
            }
            BasicButton {
                Layout.preferredHeight: 68
                Layout.preferredWidth: 168
                textLayout {
                    fillWidth: true
                    fillHeight: true
                }
                horizontalAlignment: Text.AlignLeft
                verticalAlignment: Text.AlignBottom
                Style.class: "inputMultiTextOk"
                Tr { code: "saveButton"; defaultMessage: "Сохранить" }
                onClicked: accept()
            }
        }
        // Тень
        Rectangle {
            anchors.bottom: parent.top
            width: parent.width
            height: 5
            gradient: Gradient {
                orientation: Gradient.Vertical
                GradientStop { position: 0.0; color: "transparent" }
                GradientStop { position: 1.0; color: Style.parameter("color.shadow") }
            }
            opacity: 0.3
        }
    }

    // !!!!!!!!!! Клавиатура !!!!!!!!!!
    PopupKeyboard {
        id: popupKeyboard
        keyboard {
            showCancel: !params.hideCancelButton
            layoutType: params.useDigitalKeyboard ? "input_digits" : "main"
            enterTr: Utils.makeTr("saveButton", "Сохранить")
            onHide: {
                eventReceiver.focus = false
                eventReceiver = null
            }
            onEnter: accept()
            onCancel: reject()
        }
        Connections {
            target: root
            function onEventReceiverChanged() {
                if (eventReceiver != null)
                    popupKeyboard.open()
                else
                    popupKeyboard.close()
            }
        }
    }

    DialogHandler {
        id: handler
        allowDevices: params.allowDevices
    }

    function accept() {
        let fields = Object.values(fieldsById)
        let valid = true
        let result = {}
        for (let i = 0; i < fields.length; ++i) {
            fields[i].noTries = false
            valid = valid && fields[i].validValue
            if (valid) {
                switch (fields[i].fieldType) {
                case InputMultiTextModel.String:
                    result[fields[i].objectName] = fields[i].text
                    break
                case InputMultiTextModel.Date:
                    result[fields[i].objectName] = Date.fromLocaleString(Qt.locale(), fields[i].text, fields[i].dateFormat)
                    break
                case InputMultiTextModel.DateTime:
                    result[fields[i].objectName] = fields[i].text
                    break
                case InputMultiTextModel.Choice:
                    result[fields[i].objectName] = fields[i].choiceValue
                    break
                case InputMultiTextModel.Check:
                    result[fields[i].objectName] = fields[i].checkState === Qt.Checked
                }
            }
        }
        if (!valid)
            return

        handler.sendMultiTextAnswer(params.fields, result)
    }

    function reject() {
        handler.sendAnswer({})
    }
}
