import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Qt.labs.qmlmodels

import "components"
import "view"

import Artix.Core
import Artix.Gui
import Artix.Models
import Artix.Handlers

/**
 * @brief Форма заказа
 */
BasicView {
    id: root

    property BasicTextField eventReceiver: inputField
    property FormEventFilter eventFilter: FormEventFilter {
        onCancel: () => handler.esc()
        onKeyboardData: (data, rawData, source) => { inputField.clear(); handler.ok(data) }
        onOk: input()
    }

    ItemSelectionModel {
        id: selectionModel
        model: OrderModel { id: dataModel }
    }

    function columnSizes(column) {
        switch (column) {
        case 0: return 40 + 24 + 8
        case 1: return tableView.width - columnSizes(0) - columnSizes(2) - columnSizes(3) - columnSizes(4)
        case 2: return 70 + 8 + 8
        case 3: return 110 + 8 + 8
        case 4: return 24 + 24 + 8
        }
    }

    component BasicDelegate: BasicButton {
        id: tableDelegate
        Style.class: "orderRowDelegate"
        padding: 0
        leftPadding: column === 0 ? 24 : 8
        rightPadding: column === tableView.columns - 1 ? 24 : 8
        down: pressed || selectionModel.currentIndex.row === row
        pressed: false
        textOpacity: necessaryScan ? 0.5 : 1
        Line {
            anchors.top: parent.bottom
            width: tableDelegate.width
            strokeWidth: 1
            color: Style.parameter("color.grayBorder")
        }
        MouseArea {
            anchors.fill: parent
            onPressedChanged: tableView.rowPressedChanged(row, pressed)
            onClicked: {
                if (selectionModel.currentIndex.row === row)
                    selectionModel.clearCurrentIndex()
                else
                    selectionModel.setCurrentIndex(dataModel.index(row, column), ItemSelectionModel.SelectCurrent) // выделить позицию
            }
        }
        Connections {
            target: tableView
            function onRowPressedChanged(row, pressed) {
                if (row === model.row)
                    tableDelegate.pressed = pressed
            }
        }
    }

    component TextDelegate: BasicDelegate {
        text: display
        textLayout { fillWidth: true; fillHeight: true }
        verticalAlignment: Qt.AlignVCenter
        horizontalAlignment: column === 3 ? Qt.AlignRight : Qt.AlignLeft
    }

    component ImageDelegate: BasicDelegate {
        id: imageDelegateId
        iconLayout {
            alignment: Qt.AlignVCenter
            preferredWidth: 24
            preferredHeight: 24
        }
        icon: display ? Style.parameter("icon.order.necessaryScan")
                      : Style.parameter("icon.order.scanned")
    }

    contentItem: RowLayout {
        spacing: 0
        // позиции
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                BasicLabel {
                    Style.class: "orderTitle"
                    Layout.fillWidth: true
                    Layout.leftMargin: 24
                    Layout.rightMargin: 24
                    Layout.topMargin: 16
                    Tr { code: "orderFormTitle"; defaultMessage: "Заказ №%1"; args: [dataModel.orderId] }
                }
                Item { Layout.preferredHeight: 24 }
                HorizontalHeaderView {
                    id: headerView
                    required property var model
                    Layout.fillWidth: true
                    Layout.preferredHeight: 30
                    model: dataModel
                    syncView: tableView
                    clip: true
                    resizableColumns: false
                    rowHeightProvider: function(row) { return 30 }
                    interactive: false
                    delegate: BasicLabel {
                        Style.class: "orderHeader"
                        padding: 0
                        leftPadding: column === 0 ? 24 : 8
                        rightPadding: column === tableView.columns - 1 ? 24 : 8
                        verticalAlignment: Label.AlignVCenter
                        horizontalAlignment: column === 3 ? Qt.AlignRight : Qt.AlignLeft
                        Tr { tr: display }
                    }

                    // FIXME: модель шлет сигнал modelReset, из-за чего виджет пересоздается.
                    // Но после удаления старый виджет продолжает отображаться на экране, причем в неправильной позиции.
                    // Из-за этого возникает задвоение заголовков. Установка `visible = false` при удалении - костыль
                    Component.onDestruction: () => { visible = false }
                }
                Line {
                    Layout.fillWidth: true
                    Layout.preferredHeight: strokeWidth
                    strokeWidth: 2
                    color: Style.parameter("color.grayDark")
                }
                TableView {
                    id: tableView
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    signal rowPressedChanged(int row, bool value)
                    model: dataModel
                    clip: true
                    contentWidth: width
                    columnWidthProvider: (column) => columnSizes(column)
                    rowHeightProvider: function(row) { return 88 }
                    rowSpacing: 1
                    resizableColumns: false

                    // При масштабировании, списки могут отображаться с артефактами.
                    // Рендерим с мультисемплированием в нужном разрешении. Не знаю за счет чего, но проблему это решило.
                    layer {
                        enabled: mainWindow.multisampling
                        samples: 2
                        textureSize: Qt.size(width * mainWindow.scaleFactor, height * mainWindow.scaleFactor)
                    }

                    delegate: DelegateChooser {
                        role: "contentType"
                        DelegateChoice {
                            roleValue: OrderModel.TextContent
                            delegate: TextDelegate {}
                        }
                        DelegateChoice {
                            roleValue: OrderModel.MoneyContent
                            delegate: TextDelegate {
                                text: Utils.formatMoneyItem(display)
                            }
                        }
                        DelegateChoice {
                            roleValue: OrderModel.QuantityContent
                            delegate: TextDelegate {
                                text: Utils.weightStr(display)
                            }
                        }
                        DelegateChoice {
                            roleValue: OrderModel.ImageContent
                            delegate: ImageDelegate {}
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
            }
        }
        // Цифровая клавиатура
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 0
                BasicTextField {
                    id: inputField
                    Style.class: "orderInput"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 64
                    padding: 12
                    hideCursor: true
                    fixedCursorPosition: true
                    validator: DoubleValidator { bottom: 0.0; decimals: 2 }
                    Tr on placeholderText { code: "orderInputPlaceholder"; defaultMessage: "Штрих-код" }
                }
                Item { Layout.preferredHeight: 24 }
                InlineKeyboard {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 296
                    layoutType: "digits"
                }
                Item { Layout.preferredHeight: 16 }
                BasicButton {
                    Style.class: "orderStorno"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    verticalAlignment: Qt.AlignVCenter
                    Tr { code: "orderFormStornoSelectButton"; defaultMessage: "Сторно" }
                    onClicked: handler.storno(selectionModel.currentIndex.row)
                }
                Item { Layout.preferredHeight: 8 }
                BasicButton {
                    Style.class: "orderCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "orderFormCancelButton"; defaultMessage: "Назад" }
                    onClicked: handler.esc()
                }
                Item { Layout.preferredHeight: 8 }
                BasicButton {
                    Style.class: "orderOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    icon: Style.parameter("icon.actionPanel.arrow")
                    onClicked: input()
                    Tr { code: "orderFormOkButton"; defaultMessage: "Ввод" }
                }
            }
        }
    }

    OrderHandler { id: handler }

    function input() {
        let input = inputField.text
        inputField.clear()
        handler.ok(input)
    }
}
